#include "image.h"
#include <ctype.h>

/*
 * Lecture d'une ligne du fichier.
 * On saute les lignes commençant par un "#" (commentaire)
 * On simplifie en considérant que les lignes ne dépassent pas MAXLIGNE
 */

void lire_ligne(FILE *f, char *ligne)
{
  do
    fgets(ligne,MAXLIGNE,f);
  while(ligne[0]=='#');
}

/*
 * Allocation d'une image
 */

struct image* allocation_image(int hauteur, int largeur)
{
  struct image* image;
  ALLOUER(image,1);

  image->hauteur = hauteur;
  image->largeur = largeur;
  
  ALLOUER(image->pixels,hauteur);

  int i;
  for(i=0;i<hauteur;i++)
    ALLOUER(image->pixels[i],largeur);

  return image;

}

/*
 * Libération image
 */

void liberation_image(struct image* image)
{
  int i;
  for(i=0;i<image->hauteur;i++)
    free(image->pixels[i]);
  free(image->pixels);

  free(image);
}

/*
 * Allocation et lecture d'un image au format PGM.
 * (L'entête commence par "P5\nLargeur Hauteur\n255\n"
 * Avec des lignes de commentaire possibles avant la dernière.
 */

struct image* lecture_image(FILE *f)
{
  unsigned int hauteur=0, largeur,i,j,c;
  //  unsigned char c;
  char* ligne=NULL;
  ALLOUER(ligne,1);
  
  lire_ligne(f, ligne);
  //extraitre les elements du header
  assert(!strcmp(ligne,"P5\n"));

  lire_ligne(f, ligne);  

  sscanf(ligne,"%d%d",&largeur,&hauteur);

  assert(largeur>0 && hauteur>0);

  struct image* image = allocation_image(hauteur,largeur);
  lire_ligne(f, ligne);//niveau de gris 
  for(i=0;i<hauteur;i++){
    for(j=0;j<largeur;j++){
      c=fgetc(f);
      // printf("%d ",c);
      image->pixels[i][j]=c;    
    }
    // printf("\n");
  }
  //  free(ligne);
  return image;
}

/*
 * Écriture de l'image (toujours au format PGM)
 */

void ecriture_image(FILE *f, const struct image *image)
{

  int i,j;
  char buffer[50];
  sprintf(buffer,"P5\n%d %d\n255\n",image->largeur,image->hauteur);

  fputs(buffer,f);

  for(i=0;i<image->hauteur;i++)
    for(j=0;j<image->largeur;j++)
      fputc(image->pixels[i][j],f);

}
