/**
 *
 * Copyright (C) 2013, 2014 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#include "simplex.h"


simplex *newSimplex(){
  simplex* s= NULL;
  ALLOUER(s,1);
  s->sommets[0]=NULL;
  s->sommets[1]=NULL;
  s->sommets[2]=NULL;
  s->voisins[0]=NULL;
  s->voisins[1]=NULL;
  s->voisins[2]=NULL;
  s->vertices=NULL;
  s->prev=NULL;
  s->next=NULL;
  s->sens=0;
  s->hauteur=0;
  s->stack=NULL;
  s->list=NULL;
  return s;
}

void resetSimplex(simplex* s){
  s->sommets[0]=NULL;
  s->sommets[1]=NULL;
  s->sommets[2]=NULL;
  s->voisins[0]=NULL;
  s->voisins[1]=NULL;
  s->voisins[2]=NULL;
  s->vertices=NULL;
  s->sens=0;
  s->hauteur=0;
}


void printSimplex(simplex* s){
  if(s!=NULL){
    printf("Sommets : \n");
    printVertex(s->sommets[0]);
    printVertex(s->sommets[1]);
    printVertex(s->sommets[2]);
    printf("hauteur = %lf\n",s->hauteur);
  }else
    printf("Simplex is NULL.\n");
}


int getIndice(simplex* s, struct _vertex* v){
  int i;
  for(i=0;i<3;i++)
    if(s->sommets[i] == v)
      return i;
  return -1;
}

void findAndReplaceVoisin(simplex* a, simplex* b, simplex* c){
  if(a != NULL){
    int i;
    for(i=0;i<3;i++)
      if(a->voisins[i] == b)
	a->voisins[i] = c;     
  }
}

int splitSimplex(listSimplex* queue, simplex* s){

  //vérifier qu'il y a encore des points dans le premier triangle de la file de priorité
#if HEAP>0

#else
  //  simplex* s = queue->s;
  removeHead(queue);
  simplex* next = s->next;
  /* int d = checkSimplexDirection(s); */
  /* printf("%s\n",d==CCW?"CCW":"CW"); */
#endif
  vertex* v = s->vertices;//sommet le plus élevé
  if(v!=NULL){    
    vertex* P = v;
    vertex* A = s->sommets[0];
    vertex* B = s->sommets[1];
    vertex* C = s->sommets[2];
    simplex* VA = s->voisins[0];
    simplex* VB = s->voisins[1];
    simplex* VC = s->voisins[2];


    resetSimplex(s);
    simplex* t1=s;
    simplex* t2=newSimplex();
    simplex* t3=newSimplex();

    t1->sommets[0]=P;
    t1->sommets[1]=A;
    t1->sommets[2]=B;

    t2->sommets[0]=P;
    t2->sommets[1]=B;
    t2->sommets[2]=C;


    t3->sommets[0]=P;
    t3->sommets[1]=C;
    t3->sommets[2]=A;

    addRef(P,3);
    addRef(A,1);
    addRef(B,1);
    addRef(C,1);
	    
    t1->voisins[0] = VC;
    t1->voisins[1] = t2;
    t1->voisins[2] = t3;
    
    t2->voisins[0] = VA;
    t2->voisins[1] = t3;
    t2->voisins[2] = t1;

    t3->voisins[0] = VB;
    t3->voisins[1] = t1;
    t3->voisins[2] = t2;

    //trouver le voisin du voisin qui est égale à s
    findAndReplaceVoisin(VA, s, t2);
    findAndReplaceVoisin(VB, s, t3);
    findAndReplaceVoisin(VC, s, t1);

    //répartition des points
    vertex* w = NULL;
    v = v->link->next;
    while(v!=NULL){
      w = v->link->next;
      if(vertexInSimplex(t1,v)!=DEHORS)
	addPointInSimplex(t1,v);
      else if(vertexInSimplex(t2,v)!=DEHORS)
	addPointInSimplex(t2,v);
      else//  if(vertexInSimplex(t3,v)!=DEHORS)
	addPointInSimplex(t3,v);      
      v = w;
    }

#if HEAP>0
    insert(queue,t1);
    insert(queue,t2);
    insert(queue,t3);
#endif
    //check :
    //pour chacun des triangles voisins, regarder si le point 0 est à l'interieur. Si oui flip. 
    struct _stack* stack = newStack();
    simplex* tmp = NULL;
    
    stack = push(stack,t3);
    stack = push(stack,t2);
    stack = push(stack,t1);
    while(empty(stack)==0){
      stack = pop(stack,&tmp);
      checkDelaunay(queue,tmp->voisins[0],tmp,tmp->sommets[0],&stack);
      checkDelaunay(queue,tmp->voisins[1],tmp,tmp->sommets[1],&stack);
      checkDelaunay(queue,tmp->voisins[2],tmp,tmp->sommets[2],&stack);
    }

#if HEAP==0
    //insertion après contrôle -> gain de performance
    if(next != NULL){
      insertAfter(next,t1,queue);
      moveInList(t1,queue);
      insertAfter(next,t2,queue);
      moveInList(t2,queue);
      insertAfter(next,t3,queue);
      moveInList(t3,queue);
    }else{
      insertSimplexInList(t1,queue);
      insertSimplexInList(t2,queue);
      insertSimplexInList(t3,queue);
    }
#endif
    free(stack);
    //    free(s);
  }
  return 1;
}

int checkDelaunay(listSimplex* queue, simplex* s1, simplex* s2,struct _vertex* v,struct _stack** stack){

  if(s1!=NULL/* && v!= NULL*/ && s1->stack==NULL ){
    //Si le vertex v se trouve dans le cercle => flip puis insertion des deux triangles dans la pile
    //sinon insertion du triangle dans la file de prioritée

#if TEST==1
    double d,r,x,y;
    circumcircle(s1,&r,&x,&y);
    d=sqrt( (v->coords[0]-x)*(v->coords[0]-x) + (v->coords[1]-y)*(v->coords[1]-y));
    if((fabs(d-r)>EPSILON) && d<r){

#else
      if(InCircle(s1->sommets[0],s1->sommets[1],s1->sommets[2],v) == DEDANS){
#endif
	struct _vertex** polygone=NULL;
	ALLOUER(polygone,POLYGONE_SIZE);

	int indice=0;

	if(puissance(s1->sommets[0],s1->sommets[1],v) != puissance(s1->sommets[0],s1->sommets[1],s1->sommets[2])){
	  polygone[0]=s1->sommets[0];
	  polygone[1]=v;
	  polygone[2]=s1->sommets[1];
	  polygone[3]=s1->sommets[2];
	  if(checkPolygoneConvexe(polygone,AGAUCHE))
	    indice = 3;
	}else if(puissance(s1->sommets[1],s1->sommets[2],v) != puissance(s1->sommets[1],s1->sommets[2],s1->sommets[0])){
	  polygone[0]=s1->sommets[0];
	  polygone[1]=s1->sommets[1];
	  polygone[2]=v;
	  polygone[3]=s1->sommets[2];
	  if(checkPolygoneConvexe(polygone,AGAUCHE))
	    indice = 1;
	}else if(puissance(s1->sommets[2],s1->sommets[0],v) != puissance(s1->sommets[2],s1->sommets[0],s1->sommets[1])){
	  polygone[0]=s1->sommets[0];
	  polygone[1]=s1->sommets[1];
	  polygone[2]=s1->sommets[2];
	  polygone[3]=v;
	  if(checkPolygoneConvexe(polygone,AGAUCHE))
	    indice=2;
	}
	free(polygone);
	if(!indice){
	  return 1;
	} else{
	  indice--;

	  vertex* s1_vertices =  s1->vertices;
	  vertex* s2_vertices =  s2->vertices;
	  double hs1 = s1->hauteur;
	  double hs2 = s2->hauteur;
	  
	  int indiceV = getIndice(s2,v);

	  vertex* F1A = s1->sommets[(indice+1)%3];
	  vertex* F1B = v;
	  vertex* F1C = s1->sommets[indice];

	  vertex* F2A = s1->sommets[(indice+2)%3];
	  vertex* F2B = s1->sommets[indice];;
	  vertex* F2C = v;	  
	  
	  simplex* F1VA = s2;
	  simplex* F1VB = s1->voisins[(indice+2)%3];
	  simplex* F1VC = s2->voisins[(indiceV+1)%3];
	    
	  simplex* F2VA = s1;
	  simplex* F2VB = s2->voisins[(indiceV+2)%3];
	  simplex* F2VC = s1->voisins[(indice+1)%3];

	  findAndReplaceVoisin(F2VC, s1, s2);
	  findAndReplaceVoisin(F1VC, s2, s1);
 	 
	  resetSimplex(s1);	
	  resetSimplex(s2);
	  
	  s1->sommets[0] = F1A;
	  s1->sommets[1] = F1B;
	  s1->sommets[2] = F1C;
	  s1->voisins[0] = F1VA;
	  s1->voisins[1] = F1VB;
	  s1->voisins[2] = F1VC;

	  s2->sommets[0] = F2A;
	  s2->sommets[1] = F2B;
	  s2->sommets[2] = F2C;
	  s2->voisins[0] = F2VA;
	  s2->voisins[1] = F2VB;
	  s2->voisins[2] = F2VC;

	  removeRef(s1->sommets[0],1);
	  removeRef(s2->sommets[0],1);
	  addRef(s1->sommets[1],1);
	  addRef(s1->sommets[2],1);

	  //fusionner les deux listes de points, rerépartir
	  vertex* v1 = NULL;
	  vertex* v2 = NULL;
	  v1 = s1_vertices;

	  while(v1!=NULL){
	    v2 = v1->link->next;
	    if(vertexInSimplex(s1,v1)!=DEHORS)
	      addPointInSimplex(s1,v1);
	    else// if(vertexInSimplex(s2,v1)!=DEHORS)
	      addPointInSimplex(s2,v1);	   
	    v1=v2;
	  }

	  v1 = s2_vertices;

	  while(v1!=NULL){
	    v2 = v1->link->next;
	    if(vertexInSimplex(s1,v1)!=DEHORS)
	      addPointInSimplex(s1,v1);
	    else// if(vertexInSimplex(s2,v1)!=DEHORS)
	      addPointInSimplex(s2,v1);	    
	    v1=v2;
	  }
#if HEAP>0
	  if(s1->hauteur>hs1)
	    percolateUp(queue,s1->index);
	  else
	    percolateDown(queue,s1->index);

	  if(s2->hauteur>hs2)
	    percolateUp(queue,s2->index);
	  else
	    percolateDown(queue,s2->index);

	  
	  //TODO faire un heapify précis ici
#else
	  moveInList(s1,queue);
	  moveInList(s2,queue);
#endif

	  *stack = push(*stack,s1);
	  *stack = push(*stack,s2);

	  return 0;
	}
      }
    }
    return 1;
  }



  void addPointInSimplex(simplex* s, struct _vertex* v){
    if(s->vertices==NULL){
      //premier vertex
      v->link->prev=NULL;
      v->link->next=NULL;
      s->vertices=v;
      s->hauteur = hauteur(s,v);
    }else{
      double h_tmp = hauteur(s,v);
      if(h_tmp>s->hauteur){
	//insertion au debut
	v->link->next = s->vertices;
	s->vertices->link->prev = v;
	v->link->prev = NULL;
	s->vertices = v;
	s->hauteur = h_tmp;
      }else{
	//insertion apres le premier
	v->link->next = s->vertices->link->next;
	v->link->prev = s->vertices;
	if(v->link->next != NULL)
	  v->link->next->link->prev = v;
	s->vertices->link->next = v;
      }
    }
  }


  int vertexInSimplex(simplex* s, struct _vertex* p) {
    int position1, position2,position3;
    position1 = puissance(s->sommets[0],s->sommets[1],p);
    position2 = puissance(s->sommets[1],s->sommets[2],p);
    position3 = puissance(s->sommets[2],s->sommets[0],p);

    if(position1 == position2 && position1 == position3) return DEDANS;// return position1;

    if((position1==position2 || position1 == position3 || position2==position3) && (position1 == ALIGNE || position2 == ALIGNE || position3 == ALIGNE)) return SUR;

    return DEHORS;
  }


  double hauteur(simplex* s, struct _vertex* v){
    double A = 
      (s->sommets[1]->coords[1] - s->sommets[0]->coords[1]) *
      (s->sommets[2]->coords[2] - s->sommets[0]->coords[2]) -
      (s->sommets[1]->coords[2] - s->sommets[0]->coords[2]) *
      (s->sommets[2]->coords[1] - s->sommets[0]->coords[1]);
    double B =  
      - ( (s->sommets[1]->coords[0] - s->sommets[0]->coords[0]) *
	  (s->sommets[2]->coords[2] - s->sommets[0]->coords[2]) -
	  (s->sommets[1]->coords[2] - s->sommets[0]->coords[2]) *
	  (s->sommets[2]->coords[0] - s->sommets[0]->coords[0]) );
    double C =
      (s->sommets[1]->coords[0] - s->sommets[0]->coords[0]) *
      (s->sommets[2]->coords[1] - s->sommets[0]->coords[1]) - 
      (s->sommets[1]->coords[1] - s->sommets[0]->coords[1]) *
      (s->sommets[2]->coords[0] - s->sommets[0]->coords[0]);
    double D = 
      - ( A * s->sommets[0]->coords[0] +
	  B * s->sommets[0]->coords[1] +
	  C * s->sommets[0]->coords[2] );
    double result = 
      fabs(
	   A * v->coords[0] + 
	   B * v->coords[1] + 
	   C * v->coords[2] +
	   D ) / sqrt(A*A + B*B + C*C);
    return result;
  }


  void circumcircle(simplex* s, double* radius, double* x, double* y ){
    double dA, dB, dC, aux1, aux2, div;
 
    dA = s->sommets[0]->coords[0] * s->sommets[0]->coords[0] + s->sommets[0]->coords[1] * s->sommets[0]->coords[1];
    dB = s->sommets[1]->coords[0] * s->sommets[1]->coords[0] + s->sommets[1]->coords[1] * s->sommets[1]->coords[1];
    dC = s->sommets[2]->coords[0] * s->sommets[2]->coords[0] + s->sommets[2]->coords[1] * s->sommets[2]->coords[1];
 
    aux1 = (dA*(s->sommets[2]->coords[1] - s->sommets[1]->coords[1]) + dB*(s->sommets[0]->coords[1] - s->sommets[2]->coords[1]) + dC*(s->sommets[1]->coords[1] - s->sommets[0]->coords[1]));
    aux2 = -(dA*(s->sommets[2]->coords[0] - s->sommets[1]->coords[0]) + dB*(s->sommets[0]->coords[0] - s->sommets[2]->coords[0]) + dC*(s->sommets[1]->coords[0] - s->sommets[0]->coords[0]));
    div = (2*(s->sommets[0]->coords[0]*(s->sommets[2]->coords[1] - s->sommets[1]->coords[1]) + s->sommets[1]->coords[0]*(s->sommets[0]->coords[1]-s->sommets[2]->coords[1]) + s->sommets[2]->coords[0]*(s->sommets[1]->coords[1] - s->sommets[0]->coords[1])));
 
    /* if(div == 0){  */
    /*     return false; */
    /* } */
 
    *x = aux1/div;
    *y = aux2/div;
 
    *radius = sqrt((*x - s->sommets[0]->coords[0])*(*x - s->sommets[0]->coords[0]) + (*y - s->sommets[0]->coords[1])*(*y - s->sommets[0]->coords[1]));
  }


  int left(simplex* s){
    if(s->sommets[0]->coords[0] < s->sommets[1]->coords[0] && s->sommets[0]->coords[0] < s->sommets[2]->coords[0])
      return 0;
    if(s->sommets[1]->coords[0] < s->sommets[0]->coords[0] && s->sommets[1]->coords[0] < s->sommets[2]->coords[0])
      return 1;
    return 2;
  }

  int checkSimplexDirection(simplex* s){
    int l = left(s);
    int result = puissance(s->sommets[(l-1)<0?2:l-1],s->sommets[l],s->sommets[(l+1)%3]);
    if(result == AGAUCHE) return CCW;
    else if(result == ADROITE) return CW;
    return 0;
  }




  listSimplex *newListSimplex(int capacity){
    listSimplex* l = NULL;
    ALLOUER(l,1);
    l->size=0;
#if HEAP>0
    l->capacity=capacity;
    ALLOUER(l->ss,capacity+1);
#endif
    l->s=NULL;
    l->end=NULL;
    return l;
  }

#if HEAP==0
  void printListSimplex(listSimplex* l){
    if(l!=NULL){
      simplex* s=l->s;
      while (s!=NULL){
	printf("[ %lf ; %lf ; %lf ] ",s->sommets[0]->coords[0],s->sommets[0]->coords[1],s->sommets[0]->coords[2]);
	printf("[ %lf ; %lf ; %lf ] ",s->sommets[1]->coords[0],s->sommets[1]->coords[1],s->sommets[1]->coords[2]);
	printf("[ %lf ; %lf ; %lf ] ",s->sommets[2]->coords[0],s->sommets[2]->coords[1],s->sommets[2]->coords[2]);
	printf("%lf (%d)\n",s->hauteur,sizeList(s->vertices));
	s=s->prev;
      }
    }
  }

  void insertAfter(simplex* in, simplex* new, listSimplex* l){
    new->next = in;
    new->prev = in->prev;
    if(in->prev != NULL)
      in->prev->next = new;
    in->prev = new;

  }

  void insertSimplexInList(simplex* s, listSimplex* l){
  
    s->next = NULL;
    s->prev = NULL;
    s->list=l;
    if(l->s==NULL){
      l->s=s;   
      l->end=s;
    }else{

      if(s->vertices == NULL || (s->hauteur<=l->end->hauteur && l->end->vertices != NULL)){
	if(l->end != NULL)
	  l->end->prev = s;
	s->next = l->end;
	l->end = s;
      }else{
	simplex* current = l->s;
	int ok=0;
	while(current!=NULL){
	  if(current->hauteur < s->hauteur || (current->hauteur == s->hauteur && current->vertices==NULL && s->vertices!=NULL)){
	    s->prev = current;
	    s->next = current->next;
	    if(s->next != NULL)
	      s->next->prev = s;
	    else
	      l->s=s;
	    current->next = s;
	    ok=1;
	    break;
	  }
	  if(current->prev==NULL)
	    break;
	  current = current->prev;
	}

	if(!ok){
	  s->next = current;
	  current->prev = s;
	  if(l->end==current)
	    l->end = s;
	}
      }
    }
    l->size++;
  }

  void removeSimplexFromList(simplex* s, listSimplex* l){
    if(s!=NULL && s->list==l){
      if(l->s == s)
	l->s = s->prev;	
      if(l->end==s)
	l->end = s->next;
      if(s->next!=NULL)
	s->next->prev = s->prev;
      if(s->prev!=NULL)
	s->prev->next = s->next;

      s->next = NULL;
      s->prev=NULL;
      s->list=NULL;
      l->size--;
      
    }
  }


  void removeHead(listSimplex* l){
    simplex* head = l->s;
    l->s=l->s->prev;
    head->prev=NULL;
    head->list=NULL;
    if(l->s != NULL)
      l->s->next=NULL;  
    if(l->end == head)
      l->end = NULL;
    l->size--;
  }

  void moveInList(simplex* s,listSimplex* l){
    //regarder prev, si plus petit rechercher un pour lequel on est plus grand puis insert next.
    //sinon regarder next, si plus grand rechercher un pour lequel on est plus petit puis insert prev.
     
    if((s->prev!=NULL && s->prev->hauteur > s->hauteur)  || (s->prev!=NULL && s->vertices == NULL && s->prev->vertices !=NULL)  ){
      pushBack(s,l);
    }
    if((s->next != NULL && s->next->hauteur < s->hauteur )  || ( s->next!=NULL && s->vertices != NULL && s->next->vertices ==NULL)){
      if(fabs(s->next->hauteur - s->hauteur) < fabs(l->s->hauteur - s->hauteur))	  
	pushFront(s,l);
      else 
       	insertBack(s,l); 
    }
    
  }

  void pushFront(simplex* s, listSimplex* l){

    if(s->next != NULL)
      s->next->prev = s->prev;
    else
      l->s = s->prev;

    if(s->prev != NULL)
      s->prev->next = s->next;
    else
      l->end = s->next;

    if(s->vertices == NULL){ 
      s->next = l->end;
      s->prev = NULL;
      l->end->prev = s;
      l->end = s;

    }else{
   
      simplex* tmp = NULL;

      tmp = s->next;
      while(tmp!= NULL){
	if(tmp->hauteur > s->hauteur || (tmp->hauteur == s->hauteur && tmp->vertices != NULL ))
	  break;
	tmp = tmp->next;
      }
    
    

      if(tmp==NULL){
	s->prev = l->s;
	s->next = NULL;
	l->s->next = s;
	l->s = s;
	  
      }else{
	s->prev = tmp->prev;
	s->next = tmp;
	s->prev->next = s;
	tmp->prev = s;
      }
    }  

  }
    
  void pushBack(simplex* s, listSimplex* l){

    if(s->next != NULL)
      s->next->prev = s->prev;
    else
      l->s = s->prev;

    if(s->prev != NULL)
      s->prev->next = s->next;
    else
      l->end = s->next;

    if(s->vertices == NULL){ 
      s->next = l->end;
      s->prev = NULL;
      if(l->end!=NULL)
  	l->end->prev = s;
      l->end = s;

    }else{   
      simplex* tmp=NULL;

      if(s->vertices != NULL){
	tmp = s->prev;
	while(tmp!= NULL){
	  if(tmp->hauteur < s->hauteur  || (tmp->hauteur == s->hauteur && tmp->vertices==NULL ))
	    break;
	  tmp = tmp->prev;
	}
      }
    
      if(tmp==NULL){
	s->next = l->end;
	s->prev = NULL;
	l->end->prev = s;
	l->end = s;
	  
      }else{
	s->next = tmp->next;
	s->prev = tmp;
	s->next->prev = s;
	tmp->next = s;
      }

    }
  }


  void insertBack(simplex* s, listSimplex* l){
      
    if(s->next != NULL)
      s->next->prev = s->prev;
    else
      l->s = s->prev;

    if(s->prev != NULL)
      s->prev->next = s->next;
    else
      l->end = s->next;


    simplex* tmp = l->s;
    while(tmp!= NULL){
      if(tmp->hauteur < s->hauteur)
	break;
      tmp = tmp->prev;
    }

    if(tmp==NULL){
      s->prev = l->s;
      s->next = NULL;
      l->s->next = s;
      l->s = s;
	  
    }else{
      s->prev = tmp->prev;
      s->next = tmp;
      s->prev->next = s;
      tmp->prev = s;
    }


  }

#elif HEAP>0
  void printHeap(listSimplex* l){
    int i;
    for(i = 0;i<l->size;i++){
      printf("[%d]:%d = %lf\n",i,l->ss[i]->index,l->ss[i]->hauteur);
    }

  }

  void heapify(listSimplex *l) {     
    int i;
    for (i=l->size/2; i>0; i--) 
      percolateDown(l, i);
    
  }

  void percolateDown(listSimplex *l, int index) {
    int min;
    if ((2*index+1) <= l->size) {
      min =  l->ss[2*index]->hauteur > l->ss[2*index+1]->hauteur ? 2*index : 2*index+1;
      if (l->ss[index]->hauteur < l->ss[min]->hauteur ) {
	swap(l, index, min);
	percolateDown(l, min);
      }
    }else if (l->size == 2*index) {
      if (l->ss[index]->hauteur < l->ss[2*index]->hauteur) {
	swap(l, index, 2*index);
      }
    }
  }


  void percolateUp(listSimplex *l, int index) {
    if (index > 1) {
      if (l->ss[index/2]->hauteur < l->ss[index]->hauteur ) {
	swap(l, index, index/2);
	percolateUp(l, index/2);
      }
    }
  }


  void insert(listSimplex *l, simplex* s) {
    l->size++;
    l->ss[l->size] = s;
    l->ss[l->size]->index=l->size;
    percolateUp(l, l->size);
  }

  simplex* removeMin(listSimplex *l) {
    simplex* retval=NULL;
    if (l->size > 0) {
      retval = l->ss[1];
      l->ss[1] = l->ss[l->size];
      l->ss[1]->index=1;  
      l->size--;
      percolateDown(l, 1);
    }
    retval->index=-1;
    return retval;
  }

  void swap(listSimplex *l, int index1, int index2) {
    simplex* temp = l->ss[index1];
    l->ss[index1] = l->ss[index2];
    l->ss[index2] = temp;
    l->ss[index1]->index=index1;
    l->ss[index2]->index=index2;
  }

#endif


  void deleteListSimplex(listSimplex* l){
    if(l!=NULL){
#if HEAP>0
      int i;
      for(i=0;i<l->size;i++){
	if(l->ss[i]!=NULL){


	  removeRef(l->ss[i]->sommets[0],1);
	  removeRef(l->ss[i]->sommets[1],1);
	  removeRef(l->ss[i]->sommets[2],1);

	  deleteVertex(l->ss[i]->sommets[0]);
	  deleteVertex(l->ss[i]->sommets[1]);
	  deleteVertex(l->ss[i]->sommets[2]);


	  while(l->ss[i]->vertices!=NULL){
	    vertex* v = l->ss[i]->vertices;
	    l->ss[i]->vertices = v->link->next;
	    deleteVertex(v);
	  }
	}
      }
#else
      simplex* s=l->s;
      while (s!=NULL){
	simplex* tmp = s;
	s=s->prev;
	
	removeRef(tmp->sommets[0],1);
	removeRef(tmp->sommets[1],1);
	removeRef(tmp->sommets[2],1);

	deleteVertex(tmp->sommets[0]);
	deleteVertex(tmp->sommets[1]);
	deleteVertex(tmp->sommets[2]);


	while(tmp->vertices!=NULL){
	  vertex* v = tmp->vertices;
	  tmp->vertices = v->link->next;
	  deleteVertex(v);
	}
	free(tmp);
      }
#endif
    }
    free(l);
  }
