/**
 *
 * Copyright (C) 2013, 2014 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

/**
 * \file stack.c
 * \brief GA Triangulation Delaunay
 * \author Luis Felipe Restrepo Botero, Olivier Berthier
 * \date décembre 2013
 *
 */
#ifndef STACK_H
#define STACK_H

#include "commun.h"
#include "simplex.h"

typedef struct _stack{
  struct _simplex* data;
  struct _stack* next;
  struct stack *(*newStack)();
} stack;




/*!
 * \fn stack *newStack()
 * \return stack
 * \brief Fonction d'initialisation de la pile
 */
stack *newStack();


/*! 
 * \fn struct _stack* push(struct _stack* h, simplex* d)
 * \param p pile
 * \param d simplex
 * \return stack
 * \brief Fonction permettant d'empiler un simplex sur une pile.
 */
struct _stack* push(struct _stack* p, struct _simplex* d);

/*! 
 * \fn struct _stack* pop(struct _stack* h, simplex** d)
 * \param p pile
 * \param d simplex (ref)
 * \return stack
 * \brief Fonction permettant de dépiler un simplex d'une pile.
 */
struct _stack* pop(struct _stack* p, struct _simplex** d);

/*! 
 * \fn int empty(struct _stack* p)
 * \param p pile
 * \return int
 * \brief Fonction retournant 1 si la pile est vide.
 */
int empty(struct _stack* p);


#endif
