/**
 *
 * Copyright (C) 2013, 2014 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

#include "vertex.h"

vertex *newVertex(){
  vertex* v=NULL;
  ALLOUER(v,1);
  ALLOUER(v->link,1);
  v->link->next = NULL;
  v->link->prev = NULL;
  v->compteur = 0;
  return v;
}

void deleteVertex(vertex* v){
  if(v->compteur == 0){
    free(v->link);
    free(v);
  }
}

void printList(vertex* T){
  vertex* tmp = T;
  int i=0;
  while(tmp != NULL){
    printf("%d :",i);
    printVertex(tmp);
    //    printf("\n");
    i++;
    tmp = tmp->link->next;
  }
  printf("\n");
}

int sizeList(vertex* T){
  vertex* tmp = T;
  int i=0;
  while(tmp != NULL){
    i++;
    tmp = tmp->link->next;
  }
  //  printf("SIZE : %d\n",i);
  return i;
}

void printVertex(vertex* v){
  if(v!=NULL)
   printf("[ %lf ; %lf ; %lf ]\n",v->coords[0],v->coords[1],v->coords[2]);
  else
    printf("Vertex is NULL.\n");
}


int puissance(vertex *a, vertex *b, vertex *c){
  //(Bx-Ax)*(Y-Ay) - (By-Ay)*(X-Ax)

  double result =  (b->coords[0] - a->coords[0]) * (c->coords[1] - a->coords[1]) - (b->coords[1] - a->coords[1]) * (c->coords[0] - a->coords[0]);

  if( result >= 0)
    if( result > 0)
      return AGAUCHE;
    else 
      return ALIGNE;
  else 
    return ADROITE;
    
}


int inTriangle(vertex *a, vertex *b, vertex *c, vertex* p) {
  int position1, position2,position3;
  position1 = puissance(a,b,p);
  position2 = puissance(b,c,p);
  position3 = puissance(c,a,p);

  if(position1 == position2 && position1 == position3) return DEDANS;

  if((position1==position2 || position1 == position3 || position2==position3) && (position1 == ALIGNE || position2 == ALIGNE || position3 == ALIGNE)) return SUR;

  return DEHORS;
}



int InCircle (vertex *A, vertex *B, vertex *C, vertex *Z){
    double AZx = A->coords[0] - Z->coords[0], AZy = A->coords[1] - Z->coords[1],
      BZx = B->coords[0] - Z->coords[0], BZy = B->coords[1] - Z->coords[1],
      CZx = C->coords[0] - Z->coords[0], CZy = C->coords[1] - Z->coords[1],
          
      det =
      ((A->coords[0] + Z->coords[0])*AZx +
       (A->coords[1] + Z->coords[1])*AZy) *
      (BZx*CZy - BZy*CZx) +
      ((B->coords[0] + Z->coords[0])*BZx +
       (B->coords[1] + Z->coords[1])*BZy) *
      (CZx*AZy - CZy*AZx) +
      ((C->coords[0] + Z->coords[0])*CZx +
       (C->coords[1] + Z->coords[1])*CZy) *
      (AZx*BZy - AZy*BZx);
  
  
    /* should return (det > 0.0) ? IN : (det == 0.0) ? ON : OUT; really */
    return (det > 0.0) ? DEDANS : DEHORS;
    //  return 0;
}



int checkPolygoneConvexe(struct _vertex** v, int direction){
  int result=1,i;
  for(i=0;i<POLYGONE_SIZE;i++){
    if(puissance(v[i],v[(i+1)%POLYGONE_SIZE],v[(i+2)%POLYGONE_SIZE]) != direction)
    result=0;
  }
  return result;
}

int checkPolygoneDirection(struct _vertex** v){

  int l = leftPolygone(v),i;
  int result;
  for(i=0;i<POLYGONE_SIZE;i++){
    result= puissance(v[(l-1)<0?POLYGONE_SIZE-1:(l-1)],v[l],v[(l+1)%POLYGONE_SIZE]);
    if(result != ALIGNE)
      return result;
  }
 
  return ALIGNE;
}


int leftPolygone(struct _vertex** v){
  int result = 0, i;
  struct _vertex* current = v[0];
  for(i=1;i<POLYGONE_SIZE;i++){
    if(v[i]->coords[0]<current->coords[0] ||
       (v[i]->coords[0]==current->coords[0] && v[i]->coords[1]<current->coords[1])){
      current = v[i];
      result = i;
    }
  }
  return result;
}


void crossProduct(const vertex* u, const vertex* v, vertex* r){
  r->coords[0] = u->coords[1]*v->coords[2]-u->coords[2]*v->coords[1];
  r->coords[1] = u->coords[2]*v->coords[0]-u->coords[0]*v->coords[2];
  r->coords[2] = u->coords[0]*v->coords[1]-u->coords[1]*v->coords[0];
}


void diff(const vertex* u, const vertex* v, vertex* r){
  r->coords[0] = u->coords[0] - v->coords[0];
  r->coords[1] = u->coords[1] - v->coords[1];
  r->coords[2] = u->coords[2] - v->coords[2];
}


void normalize(vertex* v){

  double length = sqrt(v->coords[0]*v->coords[0]+v->coords[1]*v->coords[1]+v->coords[2]*v->coords[2]);
   
  v->coords[0] /= length;
  v->coords[1] /= length;
  v->coords[2] /= length;
    
}

void addRef(vertex* v, int s){
  v->compteur += s;
}

void removeRef(vertex* v, int s){
  v->compteur -= s;
}
